<?php

namespace Drupal\simpleviews\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\path_alias\AliasRepositoryInterface;
use Drupal\views\Entity\View;
use Drupal\views\Views;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * SimpleViews Form.
 */
class SimpleViewsForm extends ConfigFormBase {

  /**
   * The alias repository service.
   *
   * @var \Drupal\path_alias\AliasRepositoryInterface
   */
  protected $aliasRepository;

  /**
   * Constructs a new SimpleViewsForm object.
   *
   * @param \Drupal\path_alias\AliasRepositoryInterface $alias_repository
   *   The alias repository service.
   */
  public function __construct(AliasRepositoryInterface $alias_repository) {
    $this->aliasRepository = $alias_repository;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('path_alias.repository'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'simpleviews_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    // Retrieve the view ID from the URL query parameter.
    $view_id = \Drupal::request()->query->get('view_id');

    // Load the view by ID if it exists.
    $view = $view_id ? View::load($view_id) : NULL;

    // Check if the view exists.
    if ($view) {
      // Populate the form elements with the view properties.
      $form['view_name'] = $this->createViewNameElement($view);
      $form['create_page'] = $this->createCreatePageElement($view);
      $form['page_url'] = $this->createPageUrlElement($view);
      $form['create_block'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Create views block'),
        '#default_value' => $view->getDisplay('block_1') !== NULL,
      ];
      $form['view_type'] = $this->createViewTypeElement($view);
      $form['sorted_by'] = $this->createSortedByElement($view);
      $form['display_type'] = $this->createDisplayTypeElement($view);
      $form['add_search_filter'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Add search filter'),
        '#default_value' => $this->hasSearchFilter($view),
      ];
      $form['submit_create'] = $this->createSubmitCreateElement($view);
    }
    else {
      // Add form elements to create a new view.
      $form['view_name'] = $this->createViewNameElement();
      $form['create_page'] = $this->createCreatePageElement();
      $form['page_url'] = $this->createPageUrlElement();
      $form['create_block'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Create views block'),
        '#default_value' => FALSE,
      ];
      $form['view_type'] = $this->createViewTypeElement();
      $form['sorted_by'] = $this->createSortedByElement();
      $form['display_type'] = $this->createDisplayTypeElement();
      $form['add_search_filter'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Add search filter'),
        '#default_value' => FALSE,
      ];
      $form['submit_create'] = $this->createSubmitCreateElement();
    }

    // Add states to show/hide the page_url field based on the create_page checkbox.
    $form['page_url']['#states'] = [
      'visible' => [
        ':input[name="create_page"]' => ['checked' => TRUE],
      ],
    ];

    return $form;
  }

  /**
   * Helper function to display view name.
   */
  protected function createViewNameElement($view = NULL): array {
    return [
      '#type' => 'textfield',
      '#title' => $this->t('Title'),
      '#description' => $this->t('Enter a page title for the view.'),
      '#required' => TRUE,
      '#default_value' => $view ? $view->label() : '',
    ];
  }

  /**
   * Helper function to create the "Create views page" checkbox.
   */
  protected function createCreatePageElement($view = NULL): array {
    return [
      '#type' => 'checkbox',
      '#title' => $this->t('Create views page'),
      '#default_value' => $view ? $view->getDisplay('page') !== NULL : TRUE,
    ];
  }

  /**
   * Helper function to display view url.
   */
  protected function createPageUrlElement($view = NULL) {
    // Get the current page display path if the view exists.
    $current_path = $view ? $view->getDisplay('page')['display_options']['path'] ?? '' : '';

    return [
      '#type' => 'textfield',
      '#title' => $this->t('Path'),
      '#description' => $this->t('Enter the URL path where the view will be displayed.'),
      '#default_value' => $current_path,
    ];
  }

  /**
   * Helper function to create view type select.
   */
  protected function createViewTypeElement($view = NULL) {
    $view_type = $view
      ? ($view->getDisplay('default')['display_options']['filters']['type']['value'] ?? 'all')
      : 'all';

    $options = [
        'all' => $this->t('All posts'),
      ] + $this->getViewTypes();

    return [
      '#type' => 'select',
      '#title' => $this->t('Display'),
      '#options' => $options,
      '#description' => $this->t('Select the type of content to display in the view.'),
      '#required' => TRUE,
      '#default_value' => $view_type,
    ];
  }

  /**
   * Helper function to create sorted by element.
   */
  protected function createSortedByElement($view = NULL) {
    $default_display = $view ? $view->getDisplay('default') : NULL;
    $default_sorts = $default_display && isset($default_display['display_options']['sorts']) ? $default_display['display_options']['sorts'] : [];

    $options = [
      'none' => $this->t('Unsorted'),
      'node_field_data-created:DESC' => $this->t('Newest first'),
      'node_field_data-created:ASC' => $this->t('Oldest first'),
      'node_field_data-title:ASC' => $this->t('Title'),
    ];

    $default_value = 'none';
    if (!empty($default_sorts)) {
      foreach ($default_sorts as $sort) {
        $field = $sort['field'] ?? '';
        $order = $sort['order'] ?? '';
        $sort_value = $field ? 'node_field_data-' . $field . ':' . $order : 'none';

        if (isset($options[$sort_value])) {
          $default_value = $sort_value;
          break;
        }
      }
    }

    return [
      '#type' => 'select',
      '#title' => $this->t('Sorted by'),
      '#options' => $options,
      '#default_value' => $default_value,
    ];
  }

  /**
   * Helper function to create display type select.
   */
  protected function createDisplayTypeElement($view = NULL) {
    $default_display = $view ? $view->getDisplay('default') : NULL;
    $current_row_type = $default_display ? $default_display['display_options']['row']['type'] : '';
    $current_view_mode = $default_display ? $default_display['display_options']['row']['options']['view_mode'] : '';

    $options = [
      'teaser' => $this->t('List of teasers'),
      'full' => $this->t('List of full posts'),
      'title' => $this->t('List of titles'),
    ];

    $default_value = 'teaser';

    if ($current_row_type === 'entity:node') {
      if ($current_view_mode === 'teaser') {
        $default_value = 'teaser';
      }
      elseif ($current_view_mode === 'full') {
        $default_value = 'full';
      }
    }
    elseif ($current_row_type === 'fields') {
      $default_value = 'title';
    }

    return [
      '#type' => 'select',
      '#title' => $this->t('Display as'),
      '#options' => $options,
      '#description' => $this->t('Select how the content should be displayed in the view.'),
      '#required' => TRUE,
      '#default_value' => $default_value,
    ];
  }

  /**
   * Helper function to create submit button.
   */
  protected function createSubmitCreateElement($view = NULL) {
    $view_id = \Drupal::request()->query->get('view_id');
    $value = $view_id ? $this->t('Update View') : $this->t('Create View');

    return [
      '#type' => 'submit',
      '#value' => $value,
    ];
  }

  /**
   * Helper function to check if the view has a search filter.
   */
  protected function hasSearchFilter($view): bool {
    $default_display = $view->getDisplay('default');
    return isset($default_display['display_options']['filters']['combine']);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $create_page = $form_state->getValue('create_page');
    $path = $form_state->getValue('page_url');

    if ($create_page) {
      if (empty($path)) {
        $form_state->setErrorByName('page_url', $this->t('Page URL is required when creating a views page.'));
      }
      else {
        // Clean up path first.
        $path_exists = $this->aliasRepository->lookupByAlias('/' . $path, 'en');
        if ($path_exists) {
          $form_state->setErrorByName('page_url', $this->t('This path is already in-use.'));
        }
      }
    }

    $view_id = \Drupal::request()->query->get('view_id');

    if ($view_id) {
      // If the view ID is present, it means we're editing an existing view.
      // In this case, we don't need to validate the view name or create a new view.
      return;
    }

    // Save the view using the form values.
    $view_name = $form_state->getValue('view_name');

    // Create a new view instance if it doesn't exist.
    $view_id = 'simpleviews_' . strtolower($view_name);

    // Check if the view ID already exists.
    if (Views::getView($view_id)) {
      $form_state->setErrorByName('view_name', $this->t('A view with the same name already exists.'));
    }

    // Check if the block ID exists.
    $create_block = $form_state->getValue('create_block');
    if ($create_block) {
      $block_id = 'views_block__' . $view_id . '_block_1';
      $existing_block = \Drupal::entityTypeManager()->getStorage('block')->loadByProperties(['id' => $block_id]);
      if (!empty($existing_block)) {
        $form_state->setErrorByName('create_block', $this->t('A block with this name already exists.'));
      }
    }
  }

  /**
   * {@inheritdoc}
   */

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // If there are any form errors, return early.
    if ($form_state->hasAnyErrors()) {
      return;
    }

    $create_page = $form_state->getValue('create_page');
    $create_block = $form_state->getValue('create_block');

    // Get the view ID from the query parameter.
    $view_id = \Drupal::request()->query->get('view_id');

    // Save the view using the form values.
    $view_name = $form_state->getValue('view_name');
    $view_type = $form_state->getValue('view_type');
    $page_url = $form_state->getValue('page_url');
    $view_sort = $form_state->getValue('sorted_by');
    $display_type = $form_state->getValue('display_type');
    $add_search_filter = $form_state->getValue('add_search_filter');

    // Prepare the page URL and view ID name.
    $page_url = preg_replace('/[^a-zA-Z0-9\/]+/', '-', strtolower($page_url));
    $page_url = preg_replace('/-+/', '-', $page_url);
    $page_url = trim($page_url, '-');
    $view_id_name = preg_replace('/[^a-z0-9]+/', '', $view_name);

    if ($view_id) {
      // If the view ID is present, load the existing view instance.
      $view = View::load($view_id);

      // Update the view properties.
      $view->set('label', $view_name);
    }
    else {
      // If the view ID is not present, create a new view instance.
      $view_id = 'simpleviews_' . strtolower($view_id_name);
      $view = View::create([
        'id' => $view_id,
        'label' => $view_name,
        'base_table' => 'node_field_data',
      ]);
    }

    // Setup the display before handling page and block creation/deletion.
    $this->setupDisplay($view, $page_url, $view_name, $view_type, $view_sort, $display_type, $add_search_filter, $create_page);

    // Handle page creation/deletion.
    $displays = $view->get('display');
    if ($create_page) {
      $this->createPageDisplay($view, $page_url, $view_name);
    }
    else {
      // Remove the page display if it exists.
      if (isset($displays['page'])) {
        unset($displays['page']);
        $view->set('display', $displays);
      }
    }

    // Handle block creation/deletion.
    if ($create_block) {
      $this->createBlockDisplay($view, $view_name, $view_type, $view_sort, $display_type, $add_search_filter, $create_page);
    }
    else {
      // Remove the block display if it exists.
      if (isset($displays['block_1'])) {
        unset($displays['block_1']);
        $view->set('display', $displays);
      }
    }

    // Setup the display again after page and block handling to ensure consistency.
    $this->setupDisplay($view, $page_url, $view_name, $view_type, $view_sort, $display_type, $add_search_filter, $create_page);

    if ($view) {
      $this->saveView($view, $view_name, $page_url);
      $form_state->setRedirectUrl(Url::fromRoute('simpleviews.form_list'));
    }
    else {
      $this->handleViewCreationFailure($view_name);
    }
  }

  /**
   * Helper function to save the view.
   */
  private function saveView(&$view, $view_name, $page_url) {
    try {
      // Ensure all displays are valid before saving.
      $displays = $view->get('display');
      $valid_displays = [];

      foreach ($displays as $display_id => $display) {
        // Check if the display still exists in the view configuration.
        if (isset($view->get('display')[$display_id])) {
          $valid_displays[$display_id] = $display;
        }
      }

      // Update the view with only valid displays.
      $view->set('display', $valid_displays);

      // Rebuild the executable.
      $executable = $view->getExecutable();
      $executable->destroy();
      $executable->initDisplay();

      $view->save();
      $this->messenger()->addMessage($this->t('View updated successfully: <a href="/@page_url">@view_name</a>', [
        '@page_url' => $page_url,
        '@view_name' => $view_name,
      ]));
    }
    catch (\Exception $e) {
      \Drupal::logger('simpleviews')->error('Failed to save the view: @error', ['@error' => $e->getMessage()]);
      $this->messenger()->addError($this->t('Failed to save the view. Please check the logs for more information.'));
    }
  }

  /**
   * Helper function to create the page display.
   */
  private function createPageDisplay(View &$view, $page_url, $view_name) {
    $displays = $view->get('display');

    if (!isset($displays['page'])) {
      $displays['page'] = [
        'id' => 'page',
        'display_plugin' => 'page',
        'display_title' => 'Page',
        'display_options' => [
          'display_extenders' => [],
          'path' => $page_url,
          'title' => $view_name,
        ],
      ];

      $view->set('display', $displays);

      // Rebuild the executable to include the new display.
      $executable = $view->getExecutable();
      $executable->destroy();
      $executable->initDisplay();
    }
  }

  /**
   * {@inheritdoc}
   */
  private function createBlockDisplay(View &$view, $view_name, $view_type, $view_sort, $display_type, $add_search_filter, $create_page) {
    $displays = $view->get('display');

    if (!isset($displays['block_1'])) {
      // Get the existing page URL before adding the block display.
      $page_display = $view->getDisplay('page');
      $existing_page_url = $page_display['display_options']['path'] ?? '';

      $displays['block_1'] = [
        'id' => 'block_1',
        'display_plugin' => 'block',
        'display_title' => 'Block',
        'display_options' => [
          'display_extenders' => [],
          'title' => $view_name,
        ],
      ];

      $view->set('display', $displays);

      // Rebuild the executable to include the new display.
      $executable = $view->getExecutable();
      $executable->destroy();
      $executable->initDisplay();

      $this->setupDisplay($view, $existing_page_url, $view_name, $view_type, $view_sort, $display_type, $add_search_filter, $create_page);
    }
  }

  /**
   * {@inheritdoc}
   */
  private function setupDisplay(&$view, $page_url, $view_name, $view_type, $view_sort, $display_type, $add_search_filter, $create_page) {
    $display =& $view->getDisplay('default');
    $display['display_plugin'] = 'default';
    $display['display_title'] = 'Defaults';
    $display['display_options']['title'] = $view_name;

    // Setup page display only if create_page is true.
    if ($create_page) {
      $page_display =& $view->getDisplay('page');
      if ($page_display) {
        $page_display['display_plugin'] = 'page';
        $page_display['id'] = 'page';
        $page_display['display_title'] = 'Page';
        $page_display['display_options']['path'] = $page_url;
        $page_display['display_options']['title'] = $view_name;
      }
    }

    // Apply settings to block display if it exists.
    $block_display = $view->getDisplay('block_1');
    if ($block_display) {
      $block_display['display_options']['title'] = $view_name;
    }

    // Force using fields.
    $display['display_options']['style']['options']['uses_fields'] = 'true';

    // Add the combine filter if the add search checkbox is checked.
    if ($add_search_filter) {
      $display['display_options']['filters']['combine'] = [
        'id' => 'combine',
        'table' => 'views',
        'field' => 'combine',
        'relationship' => 'none',
        'group_type' => 'group',
        'admin_label' => '',
        'plugin_id' => 'combine',
        'operator' => 'contains',
        'value' => '',
        'group' => 1,
        'exposed' => TRUE,
        'expose' => [
          'operator_id' => 'combine_op',
          'label' => 'Search',
          'description' => '',
          'use_operator' => FALSE,
          'operator' => 'combine_op',
          'operator_limit_selection' => FALSE,
          'operator_list' => [],
          'identifier' => 'title',
          'required' => FALSE,
          'remember' => FALSE,
          'multiple' => FALSE,
          'remember_roles' => [
            'authenticated' => 'authenticated',
            'anonymous' => '0',
            'viewer' => '0',
            'editor' => '0',
            'publisher' => '0',
            'webmaster' => '0',
            'administrator' => '0',
          ],
          'placeholder' => '',
        ],
        'is_grouped' => FALSE,
        'group_info' => [
          'label' => '',
          'description' => '',
          'identifier' => '',
          'optional' => TRUE,
          'widget' => 'select',
          'multiple' => FALSE,
          'remember' => FALSE,
          'default_group' => 'All',
          'default_group_multiple' => [],
          'group_items' => [],
        ],
        'fields' => [
          'title' => 'title',
        ],
      ];
    }
    else {
      unset($display['display_options']['filters']['combine']);
    }

    // Set the display options based on the selected display type.
    switch ($display_type) {
      case 'teaser':
        $display['display_options']['row']['type'] = 'entity:node';
        $display['display_options']['row']['options']['view_mode'] = 'teaser';
        break;

      case 'full':
        $display['display_options']['row']['type'] = 'entity:node';
        $display['display_options']['row']['options']['view_mode'] = 'full';
        break;

      case 'title':
        $display['display_options']['row']['type'] = 'fields';
        $display['display_options']['row']['options']['default_field_elements'] = 'true';
        $display['display_options']['row']['options']['hide_empty'] = 'false';
        $display['display_options']['row']['options']['view_mode'] = '';
        $display['display_options']['row']['options']['separator'] = '';
        break;
    }

    // Sort criteria.
    if ($view_sort === 'none') {
      unset($display['display_options']['sorts']);
    }
    else {
      $field_order = explode(':', str_replace('node_field_data-', '', $view_sort));
      [$field, $order] = $field_order;

      $display['display_options']['sorts']['created'] = [
        'id' => $field,
        'field' => $field,
        'order' => $order,
        'table' => $field ? 'node_field_data' : '',
      ];
    }

    // Title.
    $display['display_options']['fields']['title'] = [
      'id' => 'title',
      'table' => 'node_field_data',
      'field' => 'title',
      'relationship' => 'none',
      'group_type' => 'group',
      'entity_type' => 'node',
      'entity_field' => 'title',
      'plugin_id' => 'field',
    ];

    // Status.
    $display['display_options']['filters']['status'] = [
      'id' => 'status',
      'table' => 'node_field_data',
      'field' => 'status',
      'entity_type' => 'node',
      'entity_field' => 'status',
      'plugin_id' => 'boolean',
      'value' => '1',
      'group' => 1,
      'expose' => [
        'operator' => '',
      ],
    ];

    // Content type.
    if ($view_type === 'all') {
      unset($display['display_options']['filters']['type']);
    }
    else {
      $display['display_options']['filters']['type'] = [
        'id' => 'type',
        'table' => 'node_field_data',
        'field' => 'type',
        'entity_type' => 'node',
        'entity_field' => 'type',
        'plugin_id' => 'bundle',
        'value' => [
          $view_type => $view_type,
        ],
      ];
    }
  }

  /**
   * Helper function to handle view creation error.
   */
  private function handleViewCreationFailure($view_name) {
    \Drupal::logger('simpleviews')->error('Failed to create the view: @view_name', ['@view_name' => $view_name]);
    $this->messenger()->addError($this->t('Failed to create the view.'));
  }

  /**
   * Helper function to retrieve available view types.
   */
  protected function getViewTypes(): array {
    $node_types = \Drupal::entityTypeManager()->getStorage('node_type')->loadMultiple();
    $options = [];
    foreach ($node_types as $type) {
      $options[$type->id()] = $type->label();
    }
    return $options;
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [];
  }

}
